
(use-package chronometrist
  :ensure t
  :load-path "~/.emacs.d/contrapunctus/chronometrist"
  :hook
  (chronometrist-kv-read-mode . visual-line-mode)
  (kill-emacs-query-functions . chronometrist-query-stop)
  :bind ("<f9>" . #'chronometrist)
  ;; (:map chronometrist-mode-map
  ;;       ("RET"   . #'chronometrist-toggle-project-no-reason)
  ;;       ("M-RET" . #'chronometrist-toggle-project)
  ;;       ([mouse-1] . #'chronometrist-toggle-project-no-reason)
  ;;       ([mouse-3] . #'chronometrist-toggle-project))
  :config
  (require 'chronometrist-goals)
  (setq chronometrist-goals-list
        '((15  "Arrangement/new edition")
          (15  "Aural exercises")
          (15  "Transcription"
               "Music theory")
          (30  "Composing"
               "Writing"
               "Recording")
          (15  "Data organization"
               "Physical organization"
               "Khilona archiving")
          (60  "Exercise")
          (120 "Guitar")
          (90  "Reading")
          (60  "Singing")
          (20  "Subtitles")
          (15  "Acting")
          (30  "Keyboard")
          (15  "Wikisource"))
        alert-default-style 'libnotify
        chronometrist-before-in-functions  '()
        chronometrist-after-in-functions   '(;; chronometrist-tags-add
                                             ;; chronometrist-kv-add
                                             contrapunctus-start-project
                                             chronometrist-goals-run-alert-timers)
        chronometrist-before-out-functions '(contrapunctus-before-project-stop
                                             chronometrist-tags-add
                                             chronometrist-kv-add)
        chronometrist-after-out-functions  '(contrapunctus-after-project-stop
                                             chronometrist-goals-stop-alert-timers)
        chronometrist-activity-indicator
        (lambda ()
          (thread-first (chronometrist-last-expr)
            (plist-put :stop (chronometrist-format-time-iso8601))
            chronometrist-interval
            cadr
            chronometrist-seconds-to-hms
            (chronometrist-format-time "")))))

(defun contrapunctus-find-two-files-other-window (file-1 file-2)
  "Open FILE-1 and FILE-2 in new windows.
FILE-1 will appear above FILE-2."
  (find-file-other-window file-2)
  (split-window-below)
  (find-file file-1))

(defun cp-outline-open-heading (n)
  (goto-char (point-min))
  (outline-next-visible-heading n)
  (outline-show-subtree))

(defun contrapunctus-start-project (project)
  (delete-other-windows)
  (pcase project
    ("Arrangement/new edition"
     (find-file-other-window "~/Documents/Text Files/music_stuff/arrange.org"))
    ("Composing"
     (find-file-other-window "~/1-music-notation/2-my-compositions/2019/2019-03 Kahe Natak Karte Ho Ji/Kahe Natak Karte Ho Ji?.org")
     (cp-outline-open-heading 1))
    ("Data organization"
     (find-dired "~/" "-name \'dl\' -size +0c"))
    ("Exercise"
     (let* ((path-1 "~/Documents/Text Files/ma/practice.org")
            (path-2 "~/Documents/Text Files/ma/0shoto-syllabus.org")
            (file-1 (file-name-nondirectory path-1))
            (file-2 (file-name-nondirectory path-2)))
       (contrapunctus-find-two-files-other-window path-1 path-2)
       (select-window (get-buffer-window file-1))))
    ("Guitar"
     (let* ((path-1 "~/Sync/Scores/guitar-solo/repertoire.org")
            (path-2 "~/Sync/Scores/guitar-duo/repertoire.org")
            (weekday (elt (decode-time) 6))
            (week    (string-to-number (format-time-string "%U"))))
       (contrapunctus-find-two-files-other-window path-1 path-2)
       (select-window (get-buffer-window (get-file-buffer path-1)))
       (org-match-sparse-tree nil "perform")))
    ("Keyboard"
     (find-file-other-window "~/Documents/Text Files/music_stuff/piano.org")
     (outline-show-subtree))
    ("Khilona archiving"
     (find-file-other-window "/home/anon/Documents/sync/Khilona/")
     (split-window-below)
     (other-window 1)
     (find-file "/home/anon/Khilona/Videos/Me?/")
     (other-window 1)
     (find-file "/home/anon/Documents/Text Files/khilona/2011 Me?/script/script.tex"))
    ("OSM"
     ;; (async-shell-command "java -jar ~/josm-tested.jar" " *JOSM*" " *JOSM errors*")
     ;; (delete-window (get-buffer-window " *JOSM*"))
     (contrapunctus-find-two-files-other-window "~/phone/Nokia 6.1/Android/data/net.osmand.plus/files/"
                                   ;; "~/phone/Nokia 6.1/external/DCIM/OpenCamera/osm/"
                                   "~/phone/Nokia 6.1/Documents/Markor/OSM/TODO.md"))
    ;; https://github.com/bbatsov/emacs-lisp-style-guide#comment-annotations
    ("Programming"
     (grep (->> '("grep" "--color" "-EnHiIR" "-e '(TODO|FIXME|OPTIMIZE|HACK|REVIEW)'"
                  "--exclude=\"*.html\""
                  "--exclude-dir=\".git\""
                  "~/.emacs.d/contrapunctus/"
                  "~/Documents/Text\\ Files/programming/")
                (-interpose " ")
                (apply #'concat)))
     (switch-to-buffer "*grep*")
     (split-window-below)
     (other-window 1)
     (find-file "~/phone/Nokia 6.1/Documents/Markor/Computers/todo.md")
     (delete-other-windows))
    ("Singing"
     (find-file-other-window "~/Sync/Scores/voice/repertoire.org"))
    ("Subtitles"
     (find-file-other-window "~/Music/0-classical/vocal/musical/Company/"))
    ("Theatre rehearsal"
     (contrapunctus-find-two-files-other-window
      "~/Documents/Text Files/khilona/voices.org"
      "~/1-music-notation/2-my-compositions/2019/2019-03 Kahe Natak Karte Ho Ji/Kahe Natak Karte Ho Ji.org"))
    ("Transcription"
     (find-file-other-window "~/phone/Nokia 6.1/Documents/Markor/Music/transcriptions.md"))
    ("Wiktionary"
     (find-file-other-window "~/phone/Nokia 6.1/Documents/Markor/Languages/hindi.md"))
    ("Writing"
     (find-file-other-window "/home/anon/phone/Nokia 6.1/Documents/Markor/Songs or Poems/"))))

(autoload 'magit-anything-modified-p "magit")

(defun contrapunctus-commit-prompt ()
  (if (magit-anything-modified-p)
      (if (yes-or-no-p
           (concat "You have uncommitted changes in "
                   default-directory
                   ". Really clock out? "))
          t
        (magit-status)
        nil)
    t))

(defun contrapunctus-before-project-stop (project)
  (if (member project '("Composing"
                        "Khilona archiving"
                        "Programming"))
      (contrapunctus-commit-prompt)
    ;; all functions in `chronometrist-before-project-stop-functions'
    ;; must return t for successful clock-out
    t))

(require 'request)
(require 'esxml-query)
(defun contrapunctus-after-project-stop (project)
  (pcase project
    ("OSM"
     (delete-other-windows)
     ;; What should we do when there's no network connectivity?
     ;; Ideally - note the clock-out time, and retry every five
     ;; minutes. When connected, request the changesets, look for the
     ;; first changeset with a "created_at" which is less than the
     ;; clock-out time.
     (request
      "https://api.openstreetmap.org/api/0.6/changesets"
      :params '(("display_name" . "contrapunctus"))
      :parser (lambda () (libxml-parse-xml-region (point) (point-max)))
      :success
      (cl-function
       (lambda (&key data &allow-other-keys)
         (let* ((latest-changeset (-> data
                                      (esxml-node-children)
                                      (car)))
                (comment (->> latest-changeset
                              (esxml-query "[k=comment]")
                              (esxml-node-attributes)
                              (cdr)
                              (car)
                              (cdr)))
                (id (-> latest-changeset
                        (esxml-node-attributes)
                        (car)
                        (cdr))))
           (chronometrist-append-to-last-expr nil
                                              `(:osm-url ,(concat "https://www.openstreetmap.org/changeset/" id)
                                                :osm-comment ,comment)))))))
    (_ (delete-other-windows))))

(provide 'cp-chronometrist)
